`timescale 1ns/1ps

module function_f_singleHinst
#(
	parameter DATA_WIDTH = 32
)
(
input 							  clk_i			,
input 							  reset_ni      ,
input      [DATA_WIDTH-1:0] 	  R0_i		    ,
input      [DATA_WIDTH-1:0] 	  R1_i		    ,
input      [DATA_WIDTH-1:0] 	  S0_i			,
input      [DATA_WIDTH-1:0]       S1_i			,
input      [DATA_WIDTH-1:0]		  K_key_even_i	,
input      [DATA_WIDTH-1:0]		  K_key_odd_i	,

output 	   [DATA_WIDTH-1:0]  	  F0_o		    ,
output 	   [DATA_WIDTH-1:0]  	  F1_o		    	
);

`include "clog_function.vh"

wire [DATA_WIDTH-1:0] h_yield   ;
wire [DATA_WIDTH-1:0] pht_0  	;
wire [DATA_WIDTH-1:0] pht_1  	;
wire [DATA_WIDTH-1:0] R1_rol8	;

wire [DATA_WIDTH-1:0] R_selection;

reg toggle;
reg [DATA_WIDTH-1:0] h0_yield  ;
reg [DATA_WIDTH-1:0] h1_yield  ;


assign R1_rol8 = {R1_i[23:0],R1_i[(DATA_WIDTH-1):24]};

always @(posedge clk_i or negedge reset_ni) begin
	if(~reset_ni)
		toggle <= 'b0;
	else
		toggle <= ~toggle;
end

always @(posedge clk_i or negedge reset_ni) begin
	if(~reset_ni)
		h0_yield <= 'b0;
	else
	if(toggle)
		h0_yield <= h_yield;
end

always @(posedge clk_i or negedge reset_ni) begin
	if(~reset_ni)
		h1_yield <= 'b0;
	else
	if(~toggle)
		h1_yield <= h_yield;
end

assign R_selection = (toggle) ? R0_i : R1_rol8;

h_compact
#(
	.DATA_WIDTH(DATA_WIDTH)
)
h0_inst
(	
    .data_i			 (R_selection),
	.key_schedule0_i (S0_i 		 ),
	.key_schedule1_i (S1_i		 ), 
	                 
    .data_o			 (h_yield	 )
);

pht 
#(
	.DATA_WIDTH(DATA_WIDTH)
)
pht_inst
(
    .T0_i	 (h0_yield),
    .T1_i	 (h1_yield),
    .pht_o0  (pht_0	  ),
    .pht_o1  (pht_1	  )
);

assign  F0_o = (pht_0 + K_key_even_i);
assign  F1_o = (pht_1 + K_key_odd_i);

endmodule
